# This file calculates the global energy consumption of the mining industry using bottom-up data:
# - Extraction data
# - Energy intensity by mineral data

# Encoding; UTF-8

## Libraries
library(magrittr) # v2.0.3
library(tibble) # v3.2.1
library(readr) # v2.1.4
library(dplyr) # v1.1.2
library(tidyr) # v1.3.0
library(pracma) # v2.4.2
library(IEATools) # v0.1.65
library(ggplot2) # v3.4.2
library(stringr) # v1.5.0
library(purrr) # v1.0.1
library(broom) # v1.0.1
library(zoo) # v1.8.11


# Loading data ------------------------------------------------------------

## First, load extraction data

raw_extraction_data <- readr::read_csv(
  "Empty_Global_Extraction_Data.csv",
  col_types = cols(
    Mineral = col_character(),
    Unit = col_character(),
    Category = col_character(),
    `Source and Description` = col_character(),
    Group = col_character(),
    Depletion = col_character(),
    .default = col_number()
  )
)

extraction_df <- raw_extraction_data %>%
  dplyr::select(-`Source and Description`) %>%
  tidyr::pivot_longer(cols = -c("Mineral", "Unit", "Category", "Depletion", "Group"), names_to = "Year", values_to = "Extraction") %>%
  dplyr::mutate(
    Year = as.numeric(Year)
  ) %>% 
  dplyr::filter(Year > 1970)

categories <- raw_extraction_data %>% 
  dplyr::select(Mineral, Category, Depletion, Group)


## Second, load energy intensities data ##
energy_intensities_data <- read_csv("Energy_costs_factors_08_07_2022_reviewed.csv")

energy_intensities_default <- energy_intensities_data %>%
  dplyr::select(Mineral, "Extraction costs in GJ/ton", Lognormal_standev_coeff, Normal_standev_coeff) %>%
  dplyr::mutate(
    Normal_standev_coeff = as.numeric(Normal_standev_coeff)
  ) %>% 
  dplyr::rename("Constant_GJ/ton" = "Extraction costs in GJ/ton")


## Third, build a data frame combining each couple (Mineral, Year), adding also the category for each mineral.

year_mineral_df <- extraction_df %>% 
  dplyr::select(Mineral) %>% 
  dplyr::distinct() %>% 
  tidyr::expand_grid(
    Year = seq(1971, 2060)
  ) %>% 
  dplyr::left_join(categories, by = "Mineral") %>% 
  print()



# Constructing energy intensities scenarios -------------------------------

p_gradual <- 0.12/10
p_steep <- 0.66/17


# p-root strategy to smoothen the data
# https://math.stackexchange.com/questions/2931665/smooth-transition-between-any-two-functions
# This works really well.
# By defining
# h(x) = nthroot(f(x)^p + g(x)^p)
# We get a smooth function, which is always >= f(x) and => g(x)! (f and g need to be positive!)

adjustment_coeffs <- tibble(
  Year = seq(1970, 2060, 1),
  r_gradual_linear = nthroot((1 + (1 + (Year - 2015) * p_gradual)^30), 30),
  r_steep_linear = nthroot((1 + (1 + (Year - 2015) * p_steep)^10), 10),
  r_constant = 1
)



# New final_to_primary coefficient
# See PSUT calculations for its determination
final_to_primary <- 0.58

# Setting up final energy intensities - default
final_energy_intensities_default_df <- energy_intensities_default %>% 
  dplyr::mutate(
    Final_Intensity_Default = `Constant_GJ/ton` * final_to_primary
  ) %>% 
  dplyr::select(-`Constant_GJ/ton`) %>% 
  print()


# Building dynamic final energy intensities -------------------------------

# Building dynamic intensities data frame:
energy_intensities_dynamic <- energy_intensities_default %>% 
  dplyr::select(-Lognormal_standev_coeff, -Normal_standev_coeff) %>% 
  inner_join(year_mineral_df, by = c("Mineral")) %>%
  inner_join(adjustment_coeffs, by = "Year") %>%
  mutate(
    Low_Linear = case_when(
      Depletion == "N" ~ `Constant_GJ/ton`,
      Depletion == "Y" ~ `Constant_GJ/ton` * r_gradual_linear
    ),
    High_Linear = case_when(
      Depletion == "N" ~ `Constant_GJ/ton`,
      Depletion == "Y" ~ `Constant_GJ/ton` * r_steep_linear
    )
  ) %>%
  select(-c("r_gradual_linear", "r_steep_linear", "r_constant")) %>% 
  pivot_longer(cols = c("Constant_GJ/ton", "High_Linear", "Low_Linear"), 
               names_to = "Method", values_to = "Energy_GJ/ton") %>% 
  dplyr::filter(Method %in% c("Constant_GJ/ton", "High_Linear", "Low_Linear")) %>% 
  # Converting to final energy intensities:
  dplyr::mutate(
    `Energy_GJ/ton` = `Energy_GJ/ton` * final_to_primary
  ) %>% 
  print()



# Finding out energy consumption ------------------------------------------

# Finding out energy consumption by each mineral
energy_consumption_by_mineral <- dplyr::inner_join(extraction_df %>% 
                                                     dplyr::select(-Group), 
                                                   energy_intensities_dynamic, by = c("Year", "Mineral", "Category", "Depletion")) %>%
  # dplyr::left_join(
  #   TPES_TFC_UEA, by = "Year"
  # ) %>% 
  dplyr::mutate(Energy_used_GJ = Extraction * `Energy_GJ/ton`)


# Finding out total energy consumption
energy_consumption_total <- energy_consumption_by_mineral %>% 
  dplyr::group_by(Year, Method) %>% 
  dplyr::summarise(
    Total_Energy_GJ = sum(Energy_used_GJ)
  )


# Finding out the shares by each mineral
shares_energy_consumption_by_mineral <- energy_consumption_by_mineral %>% 
  dplyr::left_join(energy_consumption_total, by = c("Year", "Method")) %>% 
  dplyr::mutate(
    Share = Energy_used_GJ / Total_Energy_GJ
  ) %>% 
  print()


# Finding out energy consumption by each mineral group
energy_consumption_by_mineral_group <- energy_consumption_by_mineral %>% 
  dplyr::group_by(Year, Method, Group) %>% 
  dplyr::summarise(
    Energy_by_Group = sum(Energy_used_GJ)
  )


# Finding out the shares by each mineral group
shares_energy_consumption_by_group <- energy_consumption_by_mineral_group %>% 
  dplyr::left_join(energy_consumption_total, by = c("Year", "Method")) %>% 
  dplyr::mutate(
    Share = Energy_by_Group / Total_Energy_GJ
  ) %>% 
  print()
