# In this script we build the linear models demand = f(gdp) for each covered mineral; based on the report we've done.
# Three options:
# (1) Use the linear model obtained for the fit over the whole time period (1970-2015);
# (2) Use the lienar model obtained for the fit over the most recent time period (1995-2015);
# (3) Use the mean of the latest X values, and keep demand constant over gdps.

# Criteria:
# (1) if a fit is used, R-squared for the fit should be high enough (> 0.6);
# (2) if there is an obvious change in trend (steeper or less steep slope) in recent years, use most recent years (as soon as fit is good enough);
# (3) if no sensible fit can be assumed, do a mean of most recent values.

# Sections:
# (1) Choosing option for each mineral
# (2) Loading and tidying up data (production, intensities, wb)
# (3) Tidying up MEDEAS data
# (4) Creating demand_RoE data
# (5) Creating all linear models (demand_RoE = f(gdp))
# (6) Selecting only those models that fit what was selected in #2


# Reading and tidying up MEDEAS output data
source("sorting_medeas_data.R")


# Relevant minerals and minerals to exclude:
list_relevant_minerals <- c("Aluminium", 
                            "Cadmium",
                            "Chromium",
                            "Copper",
                            "Dysprosium",
                            "Gallium",
                            "Gravel",
                            "Indium",
                            "Iron",
                            "KNO3_mined",
                            "Limestone",
                            "Lithium",
                            "Magnesium",
                            "Manganese",
                            "Molybdenum",
                            "NaNO3_mined",
                            "Neodymium",
                            "Nickel",
                            "Lead",
                            "Sand",
                            "Silicon_sand",
                            "Silver",
                            "Tin",
                            "Sand",
                            "Sand and Gravel",
                            "Soda ash",
                            "Steel",
                            "Tellurium",
                            "Titanium",
                            "Titanium dioxide",
                            "Vanadium",
                            "Zinc",
                            "Wollastonite",
                            "Rare Earth Elements"#keep it, only needed at the end
)

list_to_exclude <- c("Iron", "Silicon sand", "Steel", "Tellurium", "Titanium", "KNO3_mined", "NaNO3_mined")




#### Choosing options for each mineral ####
# Note than the Number_past_values column is ONLY important when the method chosen is "Mean".
# Alternatively, no importance whatsoever (but for later sensitivity tests perhaps).
option_per_mineral_df <- tribble(
  ~Mineral, ~Method.chosen, ~Number_past_values,
  "Aluminium", "1970-2015", 10,
  "Antimony", "1970-2015", 10,
  'Arsenic', "Mean", 10,
  "Asbestos", "Mean", 10,
  "Barite (accounts for baryum)", "1995-2015", 10,
  "Beryllium", "Mean", 10,
  "Bismuth", "1995-2015", 10,
  "Boron", "1970-2015", 10,
  #"Bromine", "Mean", 10,
  "Cadmium", "1970-2015", 10,
  "Chromium", "1995-2015", 10,
  "Clays", "Mean", 10,
  "Cobalt", "1995-2015", 10,
  "Copper", "1970-2015", 10, # Could also be the other period. But there's no change in the linear model almost so first period.
  "Diatomite", "1970-2015", 10,
  "Feldspar", "1970-2015", 10, # Just changed to the first period - 03/12.
  "Fluorspar", "1995-2015", 10,
  "Gallium", "1995-2015", 10, # This one is really weird fit as it becomes kind of exponential in recent years. But still r.squared 0.65 so we're good.
  #"Garnet", "1970-2015", 10, # Just changed to the first period - 03/12. Not easy one.
  "Germanium", "1995-2015", 10,
  "Gold", "1970-2015", 10,
  "Graphite", "1970-2015", 10, # Just changed to the first period - 03/12.
  "Gypsum", "1970-2015", 10,
  #"Helium", "1970-2015", 10,
  "Indium", "1970-2015", 10,
  #"Iodine", "1970-2015", 10, # Just changed to the first period - 03/12.
  "Iron ore", "1970-2015", 10,
  "Kyanite", "Mean", 10,
  "Lead", "1995-2015", 10,
  "Limestone", "1970-2015", 10,
  "Lithium", "1995-2015", 10,
  "Magnesium", "1995-2015", 10,
  "Manganese", "1995-2015", 10,
  "Mercury", "Mean", 10,
  "Molybdenum", "1995-2015", 10,
  "Nickel", "1970-2015", 10,
  "Niobium", "1995-2015", 10,
  "Perlite", "1970-2015", 10,
  "Phosphate rock", "1995-2015", 10,#tricky one - chosed the highest R squared...
  "Platinum group metals", "1970-2015", 10,
  "Potash", "1995-2015", 10,
  "Pumice and Pumicite", "Mean", 10,
  "Rare Earth Elements", "1970-2015", 10,
  "Rhenium", "1995-2015", 10, # Just changed to the second period - 03/12.
  "Salt", "1970-2015", 10,
  "Sand and Gravel", "1970-2015", 10,
  "Selenium", "1970-2015", 10,
  "Silicon", "1995-2015", 10,
  "Silver", "1970-2015", 10,
  "Soda ash", "1995-2015", 10,
  "Sodium sulfate", "1970-2015", 10,
  "Strontium", "Mean", 10,# Industrial applications are declining wikipedia says.
  "Talc and Pyrophyllite", "Mean", 10,
  "Tantalum", "1970-2015", 10,
  "Tin", "Mean", 10,#tricky need to know how the industry is evolving
  "Titanium mineral concentrates", "1970-2015", 10,
  "Tungsten/Wolfram", "1995-2015", 10,
  "Uranium", "Mean", 10, # recognise that's a tricky one and we don't want to engage in forecast for uranium. Changed 03/12.
  "Vanadium", "1995-2015", 10,
  "Vermiculite", "Mean", 10,
  "Wollastonite", "Mean", 10,
  "Zinc", "1970-2015", 10,
  "Zirconium", "1970-2015", 10
)


#### Loading production, energy intensities, and WB data ####
# Raw extraction data
raw_extraction_data <- readr::read_csv("Global_Extraction_Data_08_07_2022_reviewed.csv",
                                       col_types = cols(
                                         .default = col_number(),
                                         Mineral = col_character(),
                                         `Source and Description` = col_character(),
                                         Unit = col_character(),
                                         Category = col_character(),
                                         Depletion = col_character(),
                                         Group = col_character()
                                       )) %>% 
  dplyr::select(-`Source and Description`)


# Energy costs and recycling rates data
energy_costs_data <- readr::read_csv("Energy_costs_factors_08_07_2022_reviewed.csv",
                                     col_types = cols(
                                       Mineral = col_character(),
                                       `Extraction costs in GJ/ton` = col_double(),
                                       Description = col_character(),
                                       Source_Energy_Intensity = col_character(),
                                       `Other/backing-up data and sources` = col_character(),
                                       `Recycling rate` = col_double(),
                                       `Source recycling rate` = col_character(),
                                       `Primary production costs, excl. mining` = col_double(),
                                       Source_Primary_Production = col_character(),
                                       Uncertainty_standev_coeff = col_double(),
                                       Depletion = col_character()
                                     )) %>% 
  dplyr::select(Mineral, `Extraction costs in GJ/ton`, Depletion, `Recycling rate`)


# World gdp data
raw_wb_gdp_data <- readr::read_csv("wb_gdp_constant_2010_US_dols.csv",
                                   col_types = cols(
                                     .default = col_double(),
                                     Country = col_character(),
                                     `Country Code` = col_character(),
                                     `Indicator Name` = col_character(),
                                     `Indicator Code` = col_character()
                                   ))

# Creating both production and demand historical time series
production_demand_data <- raw_extraction_data %>% 
  tidyr::pivot_longer(cols = -c("Mineral", "Unit", "Category", "Group", "Depletion"),
                      values_to = "Production",
                      names_to = "Year") %>% 
  dplyr::inner_join(energy_costs_data %>% select(-`Extraction costs in GJ/ton`), by = c("Mineral", "Depletion")) %>% 
  dplyr::mutate(
    Production = Production / 1e6,
    Demand = Production / (1 - `Recycling rate`),
    Unit = "Mt"
  ) %>% 
  print()

# Cleaning WB data
wb_gdp_data <- raw_wb_gdp_data %>% 
  dplyr::select(-Country, -`Country Code`, -`Indicator Code`) %>% 
  tidyr::pivot_longer(cols = -`Indicator Name`,
                      values_to = "Real_GDP",
                      names_to = "Year") %>% 
  dplyr::filter(Year %in% seq(1970, 2015)) %>% 
  dplyr::mutate(
    Year = as.numeric(Year),
    Real_GDP = Real_GDP / 1e12,
    `Indicator Name` = "GDP (constant 2010 TUS$)"
  ) %>% 
  dplyr::rename(
    Unit.gdp = `Indicator Name`
  ) %>% 
  print()



#### Creating demand_RoE data ####

# If needed, keep on adjusting the data here
# Do for metals that are part of the transition etc.
# First, we need to load the ratios_res_ev_to_total_for_modelling data frame by running the res_ev_to_total.R script!

# Here we change the mineral names so they fit to the ones we are using
production_demand_RoE_data <- production_demand_data %>% 
  dplyr::mutate(
    Year = as.numeric(Year)
  ) %>% 
  dplyr::left_join(historical__aggregate_minerals_res_ev_GG_100, by = c("Mineral", "Year")) %>%
  dplyr::mutate(
    Demand_RoE = case_when(
      is.na(RES_EV_tons) ~ Demand,
      is.numeric(RES_EV_tons) ~ Demand - RES_EV_tons
    ),
    ratio_demand_RoE_tot_demand = Demand_RoE / Demand
  ) %>% 
  print()


# Finding out historical demand by the rest of the economy here:
historical_demand_RoE_df <- production_demand_RoE_data %>% 
  dplyr::select(Unit.x, Mineral, Category, Year, Demand, Demand_RoE, ratio_demand_RoE_tot_demand) %>% 
  dplyr::rename(Unit = "Unit.x") %>% 
  print()


#### Creating our linear models ####

# Check https://www.r-bloggers.com/2016/09/running-a-model-on-separate-groups/
# First, fits for the period 1970-2015, for all minerals.

# (1) Creating data frame for conducting fits
data_for_linreg_1970_df <- dplyr::inner_join(historical_demand_RoE_df, wb_gdp_data, by = "Year") %>% 
  print()

# (2) Fitting and pulling all coefficients and statistics for all minerals.
glance_fits_1970_df <- data_for_linreg_1970_df %>% 
  tidyr::nest(cols = -Mineral) %>% 
  dplyr::mutate(
    fit = purrr::map(cols, ~ lm(Demand_RoE ~ Real_GDP, data = .)),
    results = purrr::map(fit, broom::glance)
  ) %>% 
  tidyr::unnest(results) %>% 
  # Now we select only what we want: r.squared, adj.r.squared, and p.value
  dplyr::select(Mineral, r.squared, adj.r.squared, p.value) %>% 
  print()


coeffs_fits_1970_df <- data_for_linreg_1970_df %>% 
  tidyr::nest(cols = -Mineral) %>% 
  dplyr::mutate(
    fit = purrr::map(cols, ~ lm(Demand_RoE ~ Real_GDP, data = .)),
    results = purrr::map(fit, broom::tidy)
  ) %>% 
  tidyr::unnest(results) %>% 
  # Now we select and tidy only what we want: estimate of each value, std.error, and p.value
  tidyr::pivot_longer(cols = c("estimate", "std.error", "statistic", "p.value"), names_to = "parameter", values_to = "value") %>% 
  dplyr::mutate(
    term = case_when(
      term == "(Intercept)" ~ "b",
      term == "Real_GDP" ~ "a"
    ),
    parameter = stringr::str_c(term, ".", parameter)
  ) %>% 
  dplyr::select(-cols, -fit, -term) %>% 
  tidyr::pivot_wider(values_from = value, names_from = parameter) %>% 
  print()


values_fits_1970_df <- inner_join(glance_fits_1970_df, coeffs_fits_1970_df, by = "Mineral") %>% 
  dplyr::mutate(
    Method.used = "1970-2015"
  ) %>% 
  dplyr::relocate(Method.used, .after = Mineral) %>%
  print()


# Second, fits for the period 1995-2015, for all minerals
# (1) Creating data frame for conducting fits
data_for_linreg_1995_df <- inner_join(historical_demand_RoE_df, wb_gdp_data, by = "Year") %>% 
  dplyr::filter(Year %in% seq(1995, 2015)) %>% 
  print()

# (2) Fitting and pulling all coefficients and statistics for all minerals.
glance_fits_1995_df <- data_for_linreg_1995_df %>% 
  tidyr::nest(cols = -Mineral) %>% 
  dplyr::mutate(
    fit = map(cols, ~ lm(Demand_RoE ~ Real_GDP, data = .)),
    results = map(fit, glance)
  ) %>% 
  tidyr::unnest(results) %>% 
  # Now we select only what we want: r.squared, adj.r.squared, and p.value
  dplyr::select(Mineral, r.squared, adj.r.squared, p.value) %>% 
  print()


coeffs_fits_1995_df <- data_for_linreg_1995_df %>% 
  tidyr::nest(cols = -Mineral) %>% 
  dplyr::mutate(
    fit = map(cols, ~ lm(Demand_RoE ~ Real_GDP, data = .)),
    results = map(fit, tidy)
  ) %>% 
  tidyr::unnest(results) %>% 
  # Now we select and tidy only what we want: estimate of each value, std.error, and p.value
  tidyr::pivot_longer(cols = c("estimate", "std.error", "statistic", "p.value"), names_to = "parameter", values_to = "value") %>% 
  dplyr::mutate(
    term = case_when(
      term == "(Intercept)" ~ "b",
      term == "Real_GDP" ~ "a"
    ),
    parameter = str_c(term, ".", parameter)
  ) %>% 
  dplyr::select(-cols, -fit, -term) %>% 
  tidyr::pivot_wider(values_from = value, names_from = parameter) %>% 
  print()


values_fits_1995_df <- dplyr::inner_join(glance_fits_1995_df, coeffs_fits_1995_df, by = "Mineral") %>% 
  dplyr::mutate(
    Method.used = "1995-2015"
  ) %>% 
  dplyr::relocate(Method.used, .after = Mineral) %>%
  print()



# Third, mean of n past values
values_fits_mean_df <- historical_demand_RoE_df %>% 
  dplyr::left_join(option_per_mineral_df) %>% 
  dplyr::mutate(
    keep_for_model = (Year - 2015 + Number_past_values) > 0
  ) %>% 
  dplyr::filter(keep_for_model == TRUE) %>% 
  dplyr::group_by(Mineral) %>% 
  dplyr::summarise(
    Method.used = "Mean",
    mean_demand_RoE = mean(Demand_RoE)
  ) %>% 
  dplyr::rename(
    b.estimate = mean_demand_RoE 
  ) %>% 
  print()


# Fourth, bind all these rows into a new data frame
values_all_fits_df <- dplyr::bind_rows(values_fits_1970_df, values_fits_1995_df, values_fits_mean_df) %>% 
  print()


#### Selecting only those models that fit the model that we have chosen in #2 ####

# Creating selected_fits_df which gather only linear models we want to keep for the analysis.
selected_fits_df <- values_all_fits_df %>% 
  dplyr::left_join(option_per_mineral_df, by = "Mineral") %>% 
  dplyr::mutate(
    to_keep = Method.used == Method.chosen
  ) %>% 
  dplyr::filter(to_keep == TRUE) %>% 
  dplyr::select(-to_keep) %>% 
  dplyr::relocate(
    Method.chosen, .after = Method.used
  ) %>% 
  dplyr::relocate(
    Number_past_values, .after = Method.chosen
  ) %>% 
  print()

rm(medeas_2019_data, medeas_2020_data, sorted_medeas_2019, sorted_medeas_2020)


# For filling in the Table in Appendix:
print(selected_fits_df)
#View(selected_fits_df)

