# This script does projections (mineral demand for RoE) for all fitted models, and then enables filtering only the chosen models.
# It assumes that the following scripts have been run first:
# (i) sorting_medeas_data.R
# (ii) build_linear_models.R
# (iii) converting_gdp_function.R

# Script decomposed as follows:
# (1) Sorting out GDP data (both from MEDEAS and from SSP1 scenario);
# (2) Linking GDP data (projections) to linear models;
# (3) Calculating demand (RoE) projection for each mineral, for each scenario, for each of the three methods;
# (4) Keeping only the chosen method.

source("converting_gdp_func.R")
source("sorting_sr_1_5_data.R")
source("load_iea_data.R")
source("degrowth_scenarios.R")


# 1. Sorting GDP data --------------------------------------------------------

# SR-1.5C scenarios. Downloaded from IIASA online database.
gdp_data_sr_1.5 <- tidy_sr_data %>% 
  dplyr::filter(Variable == "GDP|PPP", Year >= 2015) %>%
  dplyr::rename(Scenario = Model_Scenario) %>% 
  dplyr::mutate(
    Year = as.numeric(Year)
  ) %>% 
  converting_gdp(Year = "Year", Gdp_input = "Values_interpolated", Grouping_variable = "Scenario") %>% 
  print()



# Second, building the data frame (Year, Gdp_equivalent, Scnario)
gdp_data_for_projections_df <- gdp_data_sr_1.5 %>% 
  # binding degrowth scenarios here
  dplyr::bind_rows(gdp_data_degrowth) %>% 
  dplyr::select(Scenario, Year, Gdp_equivalent) %>% # If it doesnt work that's the wrong line!
  print()


# 2. Linking all that GDP projection data to the linear models we are building ---------------

building_mineral_gdp_linear_models_df <- tidyr::expand_grid(gdp_data_for_projections_df,
                                                            values_all_fits_df) %>% 
  dplyr::relocate(any_of(c("Mineral", "Scenario", "Method.used")), .before = Year) %>% 
  print()

# Note: there was also a two step process underlined here:
# (i) Joining all the gdp values to each mineral using expand_grid(), or crossing()
# (ii) Then joining with all linear models data.

list_mineral_df <- option_per_mineral_df %>% 
  dplyr::select(Mineral) #%>% 
#pull()
# Also works pulling at the end to obtain a list.

# building_mineral_gdp_df <- expand_grid(list_mineral_df,
#                                             gdp_data_for_projections_df)
# 
# building_mineral_gdp_linear_models_df_bis <- inner_join(building_mineral_gdp_df,
#                                                     values_all_fits_df,
#                                                     by = "Mineral")

# With crossing, seems to be working great too, but with different return order.
#building_mineral_gdp_df_bis <- crossing(list_mineral_df,
#                                            gdp_data_for_projections_df)



# 3. Calculating the projected mineral demand by RoE ----------------------

demand_RoE_projections <- building_mineral_gdp_linear_models_df %>% 
  dplyr::mutate(
    demand_RoE_Mtons = case_when(
      is.na(a.estimate) ~ b.estimate,
      ! is.na(a.estimate) ~ b.estimate + a.estimate * Gdp_equivalent
    )
  ) %>% 
  dplyr::relocate(demand_RoE_Mtons, .after = Year) %>% 
  print()



# 4. Selecting only those methods that we have chosen for each Mineral -----------

selected_demand_RoE_projections <- demand_RoE_projections %>% 
  dplyr::left_join(option_per_mineral_df, by = "Mineral") %>% 
  dplyr::mutate(
    to_keep = Method.used == Method.chosen
  ) %>%
  dplyr::filter(to_keep == TRUE) %>% 
  dplyr::select(-to_keep) %>% 
  dplyr::relocate(
    Method.chosen, .after = Method.used
  ) %>% 
  dplyr::relocate(
    Number_past_values, .after = Method.chosen
  ) %>% 
  print()
