# This script provides the converting_gdp() function:

# Loading libraries
library(tibble)
library(dplyr)
library(tidyr)
library(readr)


# Defining function
converting_gdp <- function(# Inputs
                           .yearly_gdp_df,
                           Year = "Year",
                           Gdp_input = "Gdp_input",
                           Grouping_variable = NULL,
                           # Outputs
                           yearly_gdp_equivalent_df,
                           Gdp_wb = "Gdp_wb",
                           Gdp_equivalent = "Gdp_equivalent"){
  
  # Checking whether value in 2015
  is_2015_provided <- .yearly_gdp_df %>% 
    filter(.data[[Year]] == 2015) %>%
    count() %>%
    pull()
  
  
  if (0 %in% is_2015_provided){
    stop("The year 2015 must be provided in the input data frame.")
  }
  
  # Picking up gdp_input values in 2015, by grouping variables
  gdp_input_values_2015_df <- .yearly_gdp_df %>% 
    ungroup() %>% # In case there are more grouping variables than there should be
    group_by_({{Grouping_variable}}) %>%  # Grouping by the grouping variable. No clue whatsoever of why the group_by_() function is needed instead of group_by()
    filter(.data[[Year]] == 2015) %>% 
    select(.data[[Gdp_input]], .data[[Grouping_variable]]) %>% 
    rename(.gdp_input_2015 = .data[[Gdp_input]])
  
  # Reading and adjusting WB GDP data
  wb_gdp_data <- read_csv("wb_gdp_constant_2010_US_dols.csv",
                              col_types = cols(
                                .default = col_double(),
                                Country = col_character(),
                                `Country Code` = col_character(),
                                `Indicator Name` = col_character(),
                                `Indicator Code` = col_character()
                              )) %>% 
    select(-Country, -`Country Code`, -`Indicator Name`, -`Indicator Code`) %>% 
    pivot_longer(cols = everything(), values_to = Gdp_wb, names_to = Year) %>%
    mutate(
      "{Gdp_wb}" := .data[[Gdp_wb]] / 1e12, # Here we decide the scale to be used for WB gdp.
      "{Year}" := as.numeric(.data[[Year]])
    )
  
  
  # Creating new yearly_gdp_equivalent_df to return
  
  gdp_wb_value_2015 <- wb_gdp_data %>% 
    filter(.data[[Year]] == 2015) %>% 
    select(.data[[Gdp_wb]]) %>% 
    pull()
  
  yearly_gdp_equivalent_df <- .yearly_gdp_df %>% 
    ungroup() %>% 
    group_by_({{Grouping_variable}}) %>% 
    left_join(wb_gdp_data, by = {{Year}}) %>% 
    left_join(gdp_input_values_2015_df, by = {{Grouping_variable}}) %>% 
    mutate(
      "{Gdp_equivalent}" := .data[[Gdp_input]] / .gdp_input_2015 * gdp_wb_value_2015
    )
  
  return(yearly_gdp_equivalent_df)
}


# Testing data frame
# 
# df_test <- tribble(
#   ~Year_input, ~gdp_input, ~case,
#   2015, 1e6, "a",
#   2016, 1e6, "a",
#   2030, 2e7, "a",
#   2035, 4e8, "a",
#   2044, 5e8, "a",
#   2058, 6e10, "a",
#   2060, 7e10, "a",
#   2015, 6e5, "b",
#   2016, 1e6, "b",
#   2030, 2e7, "b",
#   2035, 4e8, "b",
#   2044, 5e8, "b",
#   2058, 6e10, "b",
#   2060, 7e10, "b"
# )
# 
# 
# 
# df_test_converted_gdp <- df_test %>%
#   converting_gdp(Year = "Year_input", 
#                  Gdp_input = "gdp_input",
#                  Gdp_wb = "Gdp_of_wb",
#                  Gdp_equivalent = "equivalent_gdp",
#                  Grouping_variable = "case")
# 
# 
# df_test_converted_gdp_2 <- converting_gdp(.yearly_gdp_df = df_test, 
#                                           Year = "Year_input", 
#                                           Gdp_input = "gdp_input",
#                                           Gdp_wb = "Gdp_of_wb",
#                                           Gdp_equivalent = "equivalent_gdp")
# 
# 
# df_test %>% 
#   filter(Year == 2015) %>% 
#   count() %>% 
#   pull()
# 
# 
# df_test %>% 
#   filter(Year_input == 2015) %>% 
#   print()
